﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace Lines
{
    class CustomStartEndLineStyle : LineStyle
    {
		private WpfMap wpfMap;
        private PointStyle startPointStyle;
        private PointStyle endPointStyle;
        private LineStyle lineStyle;
		private int lineSize, halfSizeSymbol;

		public CustomStartEndLineStyle(WpfMap wpfMap)
			: this(wpfMap, PointSymbolType.Triangle, PointSymbolType.Triangle, GeoColor.FromArgb(255, GeoColor.StandardColors.Green), 2)
        { }

		public CustomStartEndLineStyle(WpfMap wpfMap, PointSymbolType StartPointSymbol, PointSymbolType EndPointSymbol, GeoColor color, int lineSize)
        {
			this.wpfMap = wpfMap;
			if (lineSize < 1)
				lineSize = 1;
			this.halfSizeSymbol = lineSize + 1;
			if (this.halfSizeSymbol < 4)
				this.halfSizeSymbol = 4;
			this.startPointStyle = new PointStyle(StartPointSymbol, new GeoSolidBrush(color), this.halfSizeSymbol * 2);
			this.endPointStyle = new PointStyle(EndPointSymbol, new GeoSolidBrush(color), this.halfSizeSymbol * 2);
			this.lineStyle = new LineStyle(new GeoPen(color, lineSize));
			this.lineSize = lineSize;
        }

        protected override void DrawCore(IEnumerable<Feature> features, GeoCanvas canvas, System.Collections.ObjectModel.Collection<SimpleCandidate> labelsInThisLayer, System.Collections.ObjectModel.Collection<SimpleCandidate> labelsInAllLayers)
        {
			Collection<PointShape> startPointShapes = new Collection<PointShape>();
			Collection<double> startPointShapesAngles = new Collection<double>();
			Collection<PointShape> endPointShapes = new Collection<PointShape>();
			Collection<double> endPointShapesAngles = new Collection<double>();
			Collection<BaseShape> multiLineShapes = new Collection<BaseShape>();

            foreach (Feature feature in features)
            {
                MultilineShape multilineShape = (MultilineShape)feature.GetShape();
				MultilineShape finalMultilineShape = (MultilineShape)multilineShape.CloneDeep();
                for (int i = 0; i < multilineShape.Lines.Count; i++)
                {
                    LineShape lineShape = multilineShape.Lines[i];
					PointShape startPoint = GetStartPoint(lineShape, startPointShapesAngles);
					PointShape endPoint = GetEndPoint(lineShape, endPointShapesAngles);
					if (startPoint != null)
						startPointShapes.Add(startPoint);
					if (endPoint != null)
						endPointShapes.Add(endPoint);

					LineShape finalLineShape = finalMultilineShape.Lines[i];
					if (startPoint != null)
						finalLineShape.Vertices[0] = new Vertex(startPoint);
					if (endPoint != null)
						finalLineShape.Vertices[finalLineShape.Vertices.Count - 1] = new Vertex(endPoint);
                }
				multiLineShapes.Add(finalMultilineShape);
			}

            lineStyle.Draw(multiLineShapes, canvas, labelsInThisLayer, labelsInAllLayers);
			for (int i = 0; i < startPointShapes.Count; i++)
			{
				PointShape pointShape = startPointShapes[i];
				startPointStyle.RotationAngle = (float)startPointShapesAngles[i];
				startPointStyle.Draw(new Collection<BaseShape>() { pointShape }, canvas, labelsInThisLayer, labelsInAllLayers);
			}
			for (int i = 0; i < endPointShapes.Count; i++)
			{
				PointShape pointShape = endPointShapes[i];
				endPointStyle.RotationAngle = (float)endPointShapesAngles[i];
				endPointStyle.Draw(new Collection<BaseShape>() { pointShape }, canvas, labelsInThisLayer, labelsInAllLayers);
			}
		}

		private PointShape GetStartPoint(LineShape lineShape, Collection<double> startPointShapesAngles)
		{
			LineShape resultLineShape = new LineShape();
			var vertex1 = lineShape.Vertices[0];
			var vertex2 = lineShape.Vertices[1];
			resultLineShape.Vertices.Add(vertex1);
			resultLineShape.Vertices.Add(vertex2);

			PointShape pointShape1 = new PointShape(vertex1);
			PointShape pointShape2 = new PointShape(vertex2);

			double angle = GetAngleFromTwoVertices(vertex1, vertex2);
			startPointShapesAngles.Add(angle);

			double distance = ExtentHelper.GetScreenDistanceBetweenTwoWorldPoints(
				wpfMap.CurrentExtent, pointShape1, pointShape2,
				(float)wpfMap.ActualWidth, (float)wpfMap.ActualHeight);

			float percent = (float)((this.halfSizeSymbol / distance) * 100);
			if (percent < 0 || percent > 100)
				return null;

			PointShape pointShape = resultLineShape.GetPointOnALine(StartingPoint.FirstPoint, percent);
			return pointShape;
		}

		private PointShape GetEndPoint(LineShape lineShape, Collection<double> endPointShapesAngles)
		{
			LineShape resultLineShape = new LineShape();
			var vertex1 = lineShape.Vertices[lineShape.Vertices.Count - 2];
			var vertex2 = lineShape.Vertices[lineShape.Vertices.Count - 1];
			resultLineShape.Vertices.Add(vertex1);
			resultLineShape.Vertices.Add(vertex2);

			PointShape pointShape1 = new PointShape(vertex1);
			PointShape pointShape2 = new PointShape(vertex2);

			double angle = GetAngleFromTwoVertices(vertex2, vertex1);
			endPointShapesAngles.Add(angle);

			double distance = ExtentHelper.GetScreenDistanceBetweenTwoWorldPoints(
				wpfMap.CurrentExtent, pointShape1, pointShape2,
				(float)wpfMap.ActualWidth, (float)wpfMap.ActualHeight);

			float percent = 100 - (float)((this.halfSizeSymbol / distance) * 100);
			if (percent < 0 || percent > 100)
				return null;

			PointShape pointShape = resultLineShape.GetPointOnALine(StartingPoint.FirstPoint, percent);
			return pointShape;
		}

		private double GetAngleFromTwoVertices(Vertex b, Vertex c)
		{
			double alpha = 0;
			double tangentAlpha = (c.Y - b.Y) / (c.X - b.X);
			double Peta = Math.Atan(tangentAlpha);

			if (c.X > b.X)
			{
				alpha = 90 + (Peta * (180 / Math.PI));
			}
			else if (c.X < b.X)
			{
				alpha = 270 + (Peta * (180 / Math.PI));
			}
			else
			{
				if (c.Y > b.Y) alpha = 0;
				if (c.Y < b.Y) alpha = 180;
			}
			return alpha;
		}
    }
}
